from extras.plugins import PluginConfig


class SLMConfig(PluginConfig):
    name = 'netbox_slm'
    verbose_name = 'Software Lifecycle Management'
    description = 'Software Lifecycle Management'
    version = '0.96'
    author = 'Hedde van der Heide'
    author_email = 'hedde.vanderheide@ictu.nl'
    base_url = 'slm'
    required_settings = []
    default_settings = {
        'version_info': False
    }

config = SLMConfig


from netbox.views import generic
from netbox_slm.filters import SoftwareProductFilter, SoftwareProductVersionFilter, SoftwareProductInstallationFilter
from netbox_slm.forms import (
    SoftwareProductForm, SoftwareProductFilterForm, SoftwareProductCSVForm, SoftwareProductBulkEditForm,
    SoftwareProductVersionForm, SoftwareProductVersionFilterForm, SoftwareProductVersionCSVForm,
    SoftwareProductVersionBulkEditForm, SoftwareProductInstallationForm, SoftwareProductInstallationFilterForm,
    SoftwareProductInstallationCSVForm, SoftwareProductInstallationBulkEditForm
)
from netbox_slm.models import (
    SoftwareProduct, SoftwareProductVersion, SoftwareProductInstallation
)
from netbox_slm.tables import SoftwareProductTable, SoftwareProductVersionTable, SoftwareProductInstallationTable


class SoftwareProductListView(generic.ObjectListView):
    """View for listing all existing SoftwareProducts."""

    queryset = SoftwareProduct.objects.all()
    filterset = SoftwareProductFilter
    filterset_form = SoftwareProductFilterForm
    table = SoftwareProductTable
    # template_name = "netbox_slm/object_list.html"


class SoftwareProductView(generic.ObjectView):
    """Display SoftwareProduct details"""

    queryset = SoftwareProduct.objects.all()

    def get_extra_context(self, request, instance):
        versions = instance.softwareproduct_versions.all()
        return {"versions": versions}


class SoftwareProductEditView(generic.ObjectEditView):
    """View for editing and creating a SoftwareProduct instance."""

    queryset = SoftwareProduct.objects.all()
    form = SoftwareProductForm


class SoftwareProductDeleteView(generic.ObjectDeleteView):
    """View for deleting a SoftwareProduct instance"""

    queryset = SoftwareProduct.objects.all()


class SoftwareProductBulkImportView(generic.BulkImportView):
    queryset = SoftwareProduct.objects.all()
    model_form = SoftwareProductCSVForm
    table = SoftwareProductTable


class SoftwareProductBulkEditView(generic.BulkEditView):
    queryset = SoftwareProduct.objects.all()
    filterset = SoftwareProductFilter
    table = SoftwareProductTable
    form = SoftwareProductBulkEditForm


class SoftwareProductBulkDeleteView(generic.BulkDeleteView):
    queryset = SoftwareProduct.objects.all()
    table = SoftwareProductTable


class SoftwareProductVersionListView(generic.ObjectListView):
    """View for listing all existing SoftwareProductVersions."""

    queryset = SoftwareProductVersion.objects.all()
    filterset = SoftwareProductVersionFilter
    filterset_form = SoftwareProductVersionFilterForm
    table = SoftwareProductVersionTable
    # template_name = "netbox_slm/object_list.html"


class SoftwareProductVersionView(generic.ObjectView):
    """Display SoftwareProductVersion details"""

    queryset = SoftwareProductVersion.objects.all()

    def get_extra_context(self, request, instance):
        installation_count = instance.get_installation_count()
        return {"installations": installation_count}


class SoftwareProductVersionEditView(generic.ObjectEditView):
    """View for editing and creating a SoftwareProductVersion instance."""

    queryset = SoftwareProductVersion.objects.all()
    form = SoftwareProductVersionForm


class SoftwareProductVersionDeleteView(generic.ObjectDeleteView):
    """View for deleting a SoftwareProductVersion instance"""

    queryset = SoftwareProductVersion.objects.all()


class SoftwareProductVersionBulkImportView(generic.BulkImportView):
    queryset = SoftwareProductVersion.objects.all()
    model_form = SoftwareProductVersionCSVForm
    table = SoftwareProductVersionTable


class SoftwareProductVersionBulkEditView(generic.BulkEditView):
    queryset = SoftwareProductVersion.objects.all()
    filterset = SoftwareProductVersionFilter
    table = SoftwareProductVersionTable
    form = SoftwareProductVersionBulkEditForm


class SoftwareProductVersionBulkDeleteView(generic.BulkDeleteView):
    queryset = SoftwareProductVersion.objects.all()
    table = SoftwareProductVersionTable


class SoftwareProductInstallationListView(generic.ObjectListView):
    """View for listing all existing SoftwareProductInstallations."""

    queryset = SoftwareProductInstallation.objects.all()
    filterset = SoftwareProductInstallationFilter
    filterset_form = SoftwareProductInstallationFilterForm
    table = SoftwareProductInstallationTable
    # template_name = "netbox_slm/object_list.html"


class SoftwareProductInstallationView(generic.ObjectView):
    """Display SoftwareProductInstallation details"""

    queryset = SoftwareProductInstallation.objects.all()

    # def get_extra_context(self, request, instance):
    #     records = instance.record_set.all()
    #     return {"records": records}


class SoftwareProductInstallationEditView(generic.ObjectEditView):
    """View for editing and creating a SoftwareProductInstallation instance."""

    queryset = SoftwareProductInstallation.objects.all()
    form = SoftwareProductInstallationForm


class SoftwareProductInstallationDeleteView(generic.ObjectDeleteView):
    """View for deleting a SoftwareProductInstallation instance"""

    queryset = SoftwareProductInstallation.objects.all()


class SoftwareProductInstallationBulkImportView(generic.BulkImportView):
    queryset = SoftwareProductInstallation.objects.all()
    model_form = SoftwareProductInstallationCSVForm
    table = SoftwareProductInstallationTable


class SoftwareProductInstallationBulkEditView(generic.BulkEditView):
    queryset = SoftwareProductInstallation.objects.all()
    filterset = SoftwareProductInstallationFilter
    table = SoftwareProductInstallationTable
    form = SoftwareProductInstallationBulkEditForm


class SoftwareProductInstallationBulkDeleteView(generic.BulkDeleteView):
    queryset = SoftwareProductInstallation.objects.all()
    table = SoftwareProductInstallationTable



from django.forms import BoundField
from django.urls import reverse

from utilities.forms import DynamicModelMultipleChoiceField


class CustomDynamicModelMultipleChoiceField(DynamicModelMultipleChoiceField):
    def get_bound_field(self, form, field_name):
        bound_field = BoundField(form, self, field_name)

        # Set initial value based on prescribed child fields (if not already set)
        if not self.initial and self.initial_params:
            filter_kwargs = {}
            for kwarg, child_field in self.initial_params.items():
                value = form.initial.get(child_field.lstrip("$"))
                if value:
                    filter_kwargs[kwarg] = value
            if filter_kwargs:
                self.initial = self.queryset.filter(**filter_kwargs).first()

        # Modify the QuerySet of the field before we return it. Limit choices to any data already bound: Options
        # will be populated on-demand via the APISelect widget.
        data = bound_field.value()

        if data:
            field_name = getattr(self, "to_field_name") or "pk"
            filter = self.filter(field_name=field_name)
            try:
                self.queryset = filter.filter(self.queryset, data)
            except (TypeError, ValueError):
                # Catch any error caused by invalid initial data passed from the user
                self.queryset = self.queryset.none()
        else:
            self.queryset = self.queryset.none()

        # Set the data URL on the APISelect widget (if not already set)
        widget = bound_field.field.widget
        if not widget.attrs.get("data-url"):
            app_label = self.queryset.model._meta.app_label
            model_name = self.queryset.model._meta.model_name

            #
            # Custom url for work with plugin api
            #
            data_url = reverse(
                "plugins-api:{}-api:{}-list".format(app_label, model_name)
            )
            widget.attrs["data-url"] = data_url

        return bound_field



import django_filters
from django.db.models import Q
from django.utils.translation import gettext as _
from extras.filters import TagFilter
from netbox.filtersets import NetBoxModelFilterSet
from netbox_slm.models import SoftwareProduct, SoftwareProductVersion, SoftwareProductInstallation
from utilities.forms import DynamicModelMultipleChoiceField


class BaseFilter(NetBoxModelFilterSet):
    q = django_filters.CharFilter(
        method="search",
        label="Search",
    )


class SoftwareProductFilter(BaseFilter):
    """Filter capabilities for SoftwareProduct instances."""

    name = django_filters.CharFilter(
        lookup_expr="icontains",
    )
    # tag = TagFilter()

    class Meta:
        model = SoftwareProduct
        fields = ("name",)  # "tag")

    def search(self, queryset, name, value):
        """Perform the filtered search."""
        if not value.strip():
            return queryset
        qs_filter = Q(name__icontains=value)  # | Q(status__icontains=value)
        return queryset.filter(qs_filter)


class SoftwareProductVersionFilter(BaseFilter):
    name = django_filters.CharFilter(
        lookup_expr="icontains", label="Version"
    )
    # software_product_id = django_filters.ModelMultipleChoiceFilter(
    #     queryset=SoftwareProduct.objects.all(),
    #     label='SoftwareProduct (ID)',
    # )
    # software_product = django_filters.ModelMultipleChoiceFilter(
    #     field_name='software_product__name',
    #     queryset=SoftwareProduct.objects.all(),
    #     to_field_name='name',
    #     label='SoftwareProduct (name)',
    # )
    # tag = TagFilter()

    class Meta:
        model = SoftwareProductVersion
        fields = ("name", "software_product")  # "tag")

    def search(self, queryset, name, value):
        """Perform the filtered search."""
        if not value.strip():
            return queryset
        qs_filter = Q(name__icontains=value)  # | Q(status__icontains=value)
        return queryset.filter(qs_filter)


class SoftwareProductInstallationFilter(BaseFilter):
    class Meta:
        model = SoftwareProductInstallation
        fields = tuple()  # "tag")

    def search(self, queryset, name, value):
        """Perform the filtered search."""
        if not value.strip():
            return queryset
        qs_filter = Q(software_product__name__icontains=value) | Q(version__name__icontains=value)
        return queryset.filter(qs_filter)



from django.contrib import admin
from netbox_slm.models import SoftwareProduct, SoftwareProductVersion


class SoftwareProductVersionInline(admin.TabularInline):
    model = SoftwareProductVersion
    fields = ('name',)
    extra = 0


@admin.register(SoftwareProduct)
class SoftwareProductAdmin(admin.ModelAdmin):
    list_display = ('name',)
    inlines = (SoftwareProductVersionInline,)



from django import forms
from django.urls import reverse_lazy
from django.utils.translation import gettext as _
from netbox.forms import (
    NetBoxModelForm,
    NetBoxModelCSVForm,
    NetBoxModelBulkEditForm,
    NetBoxModelFilterSetForm,
)
from dcim.models import Manufacturer, Device
from netbox_slm.models import SoftwareProduct, SoftwareProductVersion, SoftwareProductInstallation
from utilities.forms import (
    BootstrapMixin, DynamicModelChoiceField, APISelect, DynamicModelMultipleChoiceField
)
from virtualization.models import VirtualMachine


class SoftwareProductForm(NetBoxModelForm):
    """Form for creating a new SoftwareProduct object."""

    manufacturer = DynamicModelChoiceField(
        queryset=Manufacturer.objects.all(),
        required=False,
        # initial_params={
        #     'device_types': 'device_type'
        # }
    )

    class Meta:
        model = SoftwareProduct
        fields = ("name", "manufacturer", "description", "tags")


class SoftwareProductFilterForm(NetBoxModelFilterSetForm):
    """Form for filtering SoftwareProduct instances."""

    model = SoftwareProduct

    q = forms.CharField(required=False, label="Search")

    name = forms.CharField(
        required=False,
        label="Name",
    )

    # tag = TagFilterField(SoftwareProduct)


class SoftwareProductCSVForm(NetBoxModelCSVForm):
    class Meta:
        model = SoftwareProduct
        fields = ("name",)


class SoftwareProductBulkEditForm(NetBoxModelBulkEditForm):
    pk = forms.ModelMultipleChoiceField(
        queryset=SoftwareProduct.objects.all(),
        widget=forms.MultipleHiddenInput(),
    )

    class Meta:
        model = SoftwareProduct
        nullable_fields = []


class SoftwareProductVersionForm(NetBoxModelForm):
    """Form for creating a new SoftwareProductVersion object."""
    name = forms.CharField(label=_("Version"))

    software_product = DynamicModelChoiceField(
        queryset=SoftwareProduct.objects.all(),
        widget=APISelect(
            attrs={"data-url": reverse_lazy("plugins-api:netbox_slm-api:softwareproduct-list")}
        ),
    )

    class Meta:
        model = SoftwareProductVersion
        fields = ("name", "software_product", "tags")

    # def clean(self):
    #     import pdb;pdb.set_trace()
    #     return super(SoftwareProductVersionForm, self).clean()


class SoftwareProductVersionFilterForm(NetBoxModelFilterSetForm):
    """Form for filtering SoftwareProductVersion instances."""

    model = SoftwareProductVersion

    q = forms.CharField(required=False, label="Search")

    name = forms.CharField(
        required=False,
        label="Name",
    )

    # tag = TagFilterField(SoftwareProduct)


class SoftwareProductVersionCSVForm(NetBoxModelCSVForm):
    class Meta:
        model = SoftwareProductVersion
        fields = ("name",)


class SoftwareProductVersionBulkEditForm(NetBoxModelBulkEditForm):
    pk = forms.ModelMultipleChoiceField(
        queryset=SoftwareProduct.objects.all(),
        widget=forms.MultipleHiddenInput(),
    )

    class Meta:
        model = SoftwareProductVersion
        nullable_fields = []


class SoftwareProductInstallationForm(NetBoxModelForm):
    """Form for creating a new SoftwareProductInstallation object."""

    device = DynamicModelChoiceField(
        queryset=Device.objects.all(),
        required=False,
        # initial_params={
        #     'device_types': 'device_type'
        # }
    )
    virtualmachine = DynamicModelChoiceField(
        queryset=VirtualMachine.objects.all(),
        required=False,
        # initial_params={
        #     'device_types': 'device_type'
        # }
    )
    software_product = DynamicModelChoiceField(
        queryset=SoftwareProduct.objects.all(),
        required=True,
        widget=APISelect(
            attrs={"data-url": reverse_lazy("plugins-api:netbox_slm-api:softwareproduct-list")}
        ),
    )
    version = DynamicModelChoiceField(
        queryset=SoftwareProductVersion.objects.all(),
        required=True,
        widget=APISelect(
            attrs={"data-url": reverse_lazy("plugins-api:netbox_slm-api:softwareproductversion-list")}
        ),
        query_params={
            'software_product': '$software_product',
        }
    )

    class Meta:
        model = SoftwareProductInstallation
        fields = ("device", "virtualmachine", "software_product", "version", "tags")

    def clean(self):
        if not any([self.cleaned_data['device'], self.cleaned_data['virtualmachine']]):
            raise forms.ValidationError(_("Installation requires atleast one virtualmachine or device destination."))
        return super(SoftwareProductInstallationForm, self).clean()


class SoftwareProductInstallationFilterForm(NetBoxModelFilterSetForm):
    """Form for filtering SoftwareProductInstallation instances."""

    model = SoftwareProductInstallation

    q = forms.CharField(required=False, label="Search")

    # tag = TagFilterField(SoftwareProduct)


class SoftwareProductInstallationCSVForm(NetBoxModelCSVForm):
    class Meta:
        model = SoftwareProductInstallation
        fields = tuple()


class SoftwareProductInstallationBulkEditForm(NetBoxModelBulkEditForm):
    software_product = DynamicModelChoiceField(
        queryset=SoftwareProduct.objects.all(),
        required=False
    )
    version = DynamicModelChoiceField(
        queryset=SoftwareProductVersion.objects.all(),
        required=False
    )
    model = SoftwareProductInstallation
    fieldsets = (
        (None, ('software_product', 'version',)),
    )
    # nullable_fields = ('',)